use gtk4 as gtk;

use crate::{model::builder_traits::*, view::properties::*};
use gtk::{prelude::*, *};

pub type InputLabel = String;

macro_rules! build_for {
    ( $(($comp:ty,$name:ident)),* ) => {
        $(
            pub fn $name(self, input_height: Option<i32>) -> Input<$comp> {
                let input_component = Box::new(Orientation::Vertical, 0);

                let input_label = Label::builder()
                    .halign(self.align.horizontal)
                    .valign(self.align.vertical)
                    .set_margin(self.margins)
                    .label(self.label)
                    .build();

                let mut input_builder = <$comp>::builder()
                    .set_margin(MarginData::EqualsMargin(6));

                if let Some(height) = input_height {
                    input_builder = input_builder.height_request(height);
                }

                let input = input_builder.build();

                input_component.append(&input_label);
                input_component.append(&input);

                Input {
                    component: input_component,
                    input: input.clone(),
                    input_label: input_label.clone(),
                }
            }
        )*
    };
}

#[derive(Clone, Debug)]
pub struct Input<I> {
    component: Box,
    input: I,
    input_label: Label,
}

pub struct InputBuilder {
    align: Alignment,
    label: InputLabel,
    margins: MarginData,
}

impl<I> Product<InputBuilder, Box> for Input<I> {
    fn builder() -> InputBuilder {
        InputBuilder {
            align: Alignment {
                horizontal: Align::Start,
                vertical: Align::Start,
            },
            label: String::new(),
            margins: MarginData::EqualsMargin(5),
        }
    }

    fn get(&self) -> &Box {
        &self.component
    }
}

impl<I> Input<I> {
    pub fn get_component(&self) -> &Box {
        &self.component
    }

    pub fn get_input(&self) -> &I {
        &self.input
    }

    pub fn get_label(&self) -> &Label {
        &self.input_label
    }
}

impl InputBuilder {
    pub fn label(mut self, label: &str) -> Self {
        self.label = label.into();

        self
    }

    pub fn align(mut self, align: Alignment) -> Self {
        self.align = align;

        self
    }

    pub fn margins(mut self, margin: MarginData) -> Self {
        self.margins = margin;

        self
    }

    build_for!((TextView, build), (Entry, build_entry));
}
